<?php
/**
 * @version     0.0.1
 * @package     Babel-U-Lib
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Mathew Lenning - http://mathewlenning.com/
 */
// No direct access
defined('_JEXEC') or die;

abstract class Babelu_libControllerJoomlaBase implements Babelu_libControllerInterface, Serializable
{
	/**
	 * The application object.
	 *
	 * @var    JApplicationBase
	 * @since  12.1
	 */
	protected $app;
	
	/**
	 * The input object.
	 *
	 * @var    JInput
	 * @since  12.1
	 */
	protected $input;

	/**
	 * Configuration variables
	 * @var array
	 */
	protected $config;
	
	/**
	 * Associative array of models
	 * stored as $models[$prefix][$name] used by get models
	 * @var array
	 */
	protected $models = array();

	/**
	 * URL for redirection.
	 *
	 * @var    string
	 * @since  12.2
	 * @note   Replaces _redirect.
	 */
	protected $redirect;
		
	/**
	 * Redirect message.
	 *
	 * @var    string
	 * @since  12.2
	 * @note   Replaces _message.
	 */
	protected $message;

	/**
	 * Redirect message type.
	 *
	 * @var    string
	 * @since  12.2
	 * @note   Replaces _messageType.
	 */
	protected $messageType;
	
	/**
	 * Instantiate the controller.
	 *
	 * @param   JInput            $input  The input object.
	 * @param   JApplicationBase  $app    The application object.
	 * @param   array             $config Configuration
	 * @since  12.1
	 */
	public function __construct(JInput $input, $app = null, $config = array())
	{
		// Setup dependencies.
		if (!($app instanceof JApplicationBase))
		{
			$app = JFactory::getApplication();
		}
		
		$config['option'] = $input->get('option');
		
		$this->app = $app;
		$this->input = $input;
		$this->config = $config;
	}
	
	/**
	 * Method to check the session token
	 */
	protected function validateSession()
	{
		JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));
	}
	
	/**
	 * Method to refresh the session token to prevent the back button
	 */
	protected function refreshToken()
	{
		$session = JFactory::getSession();
		$session->getToken(true);
	}

	/**
	 * Method to get the option prefix from the input
	 * @return string
	 */
	protected function getPrefix()
	{
		$prefix = ucfirst(substr($this->config['option'], 4));
		
		return $prefix;
	}
	
	/**
	 * Method to load a model
	 * @param string $prefix 
	 * @param string $name
	 * @param array $config
	 * @return boolean
	 */
	public function getModel($prefix, $name, $config = array())
	{
		
		$prefix = ucfirst($prefix);
		$name = ucfirst($name);
		
		
		if (isset($this->models[$prefix][$name]))
		{	
			return $this->models[$prefix][$name];
		}
		
		$class = $prefix.'Model'.$name;
		
		if (!class_exists($class))
		{
			JLog::add(JText::sprintf('JLIB_APPLICATION_ERROR_MODELCLASS_NOT_FOUND', $class), JLog::WARNING, 'jerror');
			return false;
		}
		
		$this->models[$prefix][$name] = new $class($config);
		
		return $this->models[$prefix][$name];
	}

	/**
	 * Redirects the browser or returns false if no redirect is set.
	 *
	 * @return  boolean  False if no redirect exists.
	 *
	 * @since   12.2
	 */
	public function redirect()
	{
		if ($this->hasRedirect())
		{
			$app = $this->app;
	
			// Enqueue the redirect message
			$app->enqueueMessage($this->message, $this->messageType);
	
			// Execute the redirect
			$app->redirect($this->redirect);
		}
	
		return false;
	}
	
	/**
	 * Method to check if the controller has a redirect
	 * @return boolean
	 */
	public function hasRedirect()
	{
		if (!empty($this->redirect))
		{
			return true;
		}
		return false;
	}
	
	/**
	 * Set a URL for browser redirection.
	 *
	 * @param   string  $url   URL to redirect to.
	 * @param   string  $msg   Message to display on redirect. Optional.
	 * @param   string  $type  Message type. Optional, defaults to 'message'.
	 *
	 * @return  Babelu_libControllerJoomlaBase  This object to support chaining.
	 *
	 * @since   12.2
	 */
	public function setRedirect($url, $msg = null, $type = 'message', $useJRoute = true)
	{
		if ($useJRoute)
		{
			$this->redirect = JRoute::_($url, false);
		}
		else
		{
			$this->redirect = $url;
		}
		
		if ($msg !== null)
		{
			$this->message = $msg;
		}
	
		$this->messageType = $type;

		return $this;
	}
	

	/**
	 * Method to authorise the current user for an action.
	 * This method uses the final prefix
	 * @param string $action
	 * @param string $assetName
	 */
	final protected function canDo(Babelu_libModelJoomlaBase $model, $action = 'core.manage', $assetName = null)
	{
		return $model->allowAction($action, $assetName);
	}
	
	/**
	 * Method to convert all cid array values to integer values 
	 * @param array $cid
	 * @return array $cleanCid
	 */
	protected function cleanCid($cid)
	{
		$cleanCid = array();
		foreach ((array)$cid AS $id)
		{
			$cleanCid[] = (int)$id;
		}
		return $cleanCid;
	}
	
	protected function translate($string)
	{
		return JText::_($string);
	}
	
//--------------------- Serializable interface -----------------------------------//	
	/**
	 * Serialize the controller.
	 *
	 * @return  string  The serialized controller.
	 *
	 * @since   12.1
	 */
	public function serialize()
	{
		return serialize($this->input);
	}
	
	/**
	 * Unserialize the controller.
	 *
	 * @param   string  $input  The serialized controller.
	 *
	 * @return  Babelu_libControllerJoomlaBase  Supports chaining.
	 *
	 * @since   12.1
	 * @throws  UnexpectedValueException if input is not the right class.
	 */
	public function unserialize($serialized)
	{
		// Unserialize the input.
		$this->input = unserialize($serialized);
	
		if (!($this->input instanceof JInput))
		{
			throw new UnexpectedValueException(sprintf('%s::unserialize would not accept a `%s`.', get_class($this), gettype($this->input)));
		}
	
		return $this;
	}
}